/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */
#include <Controller.h>
#include "DualSwitchSpectroController.h"
#include "controllers/common/family/Families.h"
#include "InstrumentAbstraction/Base/AbstractIOController.h"
#include "Utilities/Counter.h"
#include "ics/DeviceContainer/Def/IODef.h"

namespace d22special {

const std::string DualSwitchSpectroController::TYPE = "dualSwitchSpectro";
const int32 DualSwitchSpectroController::SLEEP_TIME_MICROSEC = 100; //0.0001 sec

DualSwitchSpectroController::DualSwitchSpectroController(const std::string& name) :
		ExperimentController(name), controller::Stoppable(this) {

	setFamily(family::SAMPLE_ENVIRONMENT);

	//init properties
	integrationTime.init(this, SAVE, "integration_time");
	numberOfCycles.init(this, SAVE, "number_cycles");
	executionStatus.init(this, SAVE, "execution_status");

	xData.init(this, SAVE, "x_data");
	xLeftData.init(this, SAVE, "x_left_data");
	yLeftData.init(this, SAVE, "y_leftData");
	yRightData.init(this, SAVE, "y_rightData");

	durationLeft.init(this, NOSAVE, "duration_left");
	durationRight.init(this, NOSAVE, "duration_right");

	m_StopActivated = false;
	m_yRightData = 0;
	m_yLeftData = 0;
	m_InitialTime = 0;

	//It has attached one module (QE65000 controller) and one controller (switcher)
	spectroController.init(this, "spectro");
	switcherController.init(this, "swicther");

}

DualSwitchSpectroController::~DualSwitchSpectroController() {

}

void DualSwitchSpectroController::postConfiguration() {
	registerProgression(spectroController, &DualSwitchSpectroController::updateProgression, this);
}

void DualSwitchSpectroController::start() {

	//init necessary stuff
	initArrays();

	m_StopActivated = false;
	commandProgression = 0;

	//Before measuring spectrum check if the spectrometer is plugged in
	if (spectroController->isConnected()) {
		executionStatus.update("idle");

		//Set up values of spectro
		spectroController->integrationTime = integrationTime();

		// To get the requires timestamps
		common::Duration duration;

		int32 cyclesNumber = numberOfCycles();
		while (cyclesNumber > 0 && !m_StopActivated) {

			commandProgression = (numberOfCycles() - cyclesNumber) * 100 / numberOfCycles();
//			common::Date initialTime = common::Date();

			common::Date initialTime = m_InitialTime;

			if (!m_StopActivated) {
				// First: start switcher -> we begin for OPEN position (== IN)
				cout << "starting left" << endl;
				executionStatus.update("Measuring Left");
				switcherController->setOpen();
				switcherController->startCommand();
				usleep(SLEEP_TIME_MICROSEC); // introduce a small sleep to make sure the switcher is in the correct channel

				//Second: start spectro
				spectroController->start();
				updateData();
			}
			common::Date leftTime = common::Date();
			duration = leftTime - initialTime;
			durationLeft = duration.getMilliseconds();

			if (!m_StopActivated) {
				//Again same process: start switcher (now to CLOSE == OUT) and then spectro
				cout << "starting right" << endl;
				executionStatus.update("Measuring Right");
				switcherController->setClose();
				switcherController->startCommand();
				usleep(SLEEP_TIME_MICROSEC); // introduce a small sleep to make sure the switcher is in the correct channel

				spectroController->start();
				updateData();
			}
			common::Date rightTime = common::Date();
			duration = rightTime - leftTime;
			durationRight = duration.getMilliseconds();

			cyclesNumber = cyclesNumber - 1;

		}
		commandProgression = 100;
		executionStatus.update("finished");

	} else {
		cout << "spectro disconnected" << endl;
		executionStatus.update("disconnected");
	}
}

void DualSwitchSpectroController::stop() {
	m_StopActivated = true;
	commandProgression = 100;
}

void DualSwitchSpectroController::updateProgression() {
//	int32 progression = 0;
//	commandProgression = progression;
}

void DualSwitchSpectroController::updateData() {

	int32 size = spectroController->size();

	if (executionStatus() == "Measuring Right") {
		xData.update(spectroController->xData());
		xData.setSize(size);
		xData.sendEvent();

		float64* tempYData = spectroController->yData();
		for (int32 i = 0; i < size; i++) {
			m_yRightData[i] = tempYData[i];
		}
		yRightData.update(m_yRightData);
		yRightData.setSize(size);
		yRightData.sendEvent();

	} else {
		//testing two x arrays of data
		xLeftData.update(spectroController->xData());
		xLeftData.setSize(size);
		xLeftData.sendEvent();

		float64* tempYData = spectroController->yData();
		for (int32 i = 0; i < size; i++) {
			m_yLeftData[i] = tempYData[i];
		}
		yLeftData.update(m_yLeftData);
		yLeftData.setSize(size);
		yLeftData.sendEvent();
	}

}

void DualSwitchSpectroController::initArrays() {
	if (m_yRightData) {
		delete[] m_yRightData;
	} else if (m_yLeftData) {
		delete[] m_yLeftData;
	}

//	int32 totalSize = 1044;
//	cout << "init arrays: " << spectroController->size() << endl;

	int32 size = spectroController->size();

	m_yRightData = new float64[size];
	memset(m_yRightData, 0, size * sizeof(float64)); //set to zero

	m_yLeftData = new float64[size];
	memset(m_yLeftData, 0, size * sizeof(float64)); //set to zero

}

void DualSwitchSpectroController::setInitialTime(common::Date initialTime) {
	m_InitialTime = initialTime;
}

}

