/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CAENADCCONTROLLER_H
#define CAENADCCONTROLLER_H

#include <Controller.h>
#include <controllers/common/acquisition/mode/ListModeProperties.h>

namespace npp {

/*!
 * \brief Command manual move to be used as a thread.
 */
class ClearData: private controller::Command {

public:
	ClearData(ExperimentController * c) :
		controller::Command(c, "clearData", &ClearData::clearData, this) {
	}

	virtual void clearData() = 0;
	void clearDataCommand(bool logging = false) {
		command(logging, true);
	}
	void clearDataParallel(bool logging = false) {
		commandParallel(logging, true);
	}
};

/*!
 * \class CAENV1724Driver
 * \brief Main class for the CAENV1724Driver device driver
 *
 * \par
 * Class for CAENV1724Driver driver. Implements DeviceDriver methods.
 * \par
 * The CAENV1724Driver device manages the global functionality of the CAEN V1724 VME Digitiser.
 */


class CAENAdcController: public ExperimentController, public ClearData, public acquisition::ListModeProperties {

public:

	//! Driver type value
	static const std::string TYPE;

	/*!
	 * \brief Constructor
	 * \param[in] name the name of the device driver
	 */
	CAENAdcController(const std::string& name);

	/*!
	 * \brief Destructor
	 */
	virtual ~CAENAdcController();

	// Global properties
	Property<std::string> modelName;
	Property<int32> acqBoardIndex;
	Property<int32> channels;
	Property<int32> nbChannels;
    Property<int32> dppAcqMode;
    Property<int32> recordLength;

	// Channels properties
	Property<int32> minRange;
	Property<int32> maxRange;

	// Channels properties outside dppParam
	Property<int32> channelActive;
	Property<int32> dcOffset;
	Property<int32> pulsePolarity;
	Property<int32> preTrigger;

	// Scope Curves
	ArrayProperty<int32> curve0;
	ArrayProperty<int32> curve1;
	ArrayProperty<int32> curve2;
	ArrayProperty<int32> curve3;
	ArrayProperty<int32> curve4;

protected:

	virtual void refreshNbChannelsProperty(int32 aValue) throw (CannotSetValue) = 0;
	virtual void refreshDisplayChannelProperty(int32 aValue) throw (CannotSetValue) = 0;
	virtual void refreshRecordLengthProperty(int32 aValue) throw (CannotSetValue) = 0;
	virtual void refreshDppAcqModeProperty(int32 aValue) throw (CannotSetValue);

	virtual void refreshChannelActiveProperty(int32 aValue) throw (CannotSetValue) = 0;

	virtual void updateBoardProperties() = 0;

	virtual void writeParam() = 0;
	virtual void initChannel(int32 aChannel) = 0;

};

}
#endif //CAENADCCONTROLLER_H
