/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <math.h>

#include "Dfk31Au03Driver.h"
#include "Dfk31Au03State.h"
#include "RealDfk31Au03Driver.h"
#include "PerfectDfk31Au03Driver.h"
#include "Dfk31Au03Def.h"

using namespace std;

namespace dfk41au03 {

const std::string Dfk31Au03Driver::TYPE = "Dfk31Au03";

/*
 * Constructor
 */
Dfk31Au03Driver::Dfk31Au03Driver(const string& name) :
		acquisition::NoTofAcquisitionCommon(name), acquisition::DetectorCommon(name) {

	registerStates(new RealDfk31Au03Driver(this), new PerfectDfk31Au03Driver(this), new PerfectDfk31Au03Driver(this));

	RGBData.init(this, NOSAVE, "RgbData");
	height.init(this, SAVE, "height");
	width.init(this, SAVE, "width");
	pixelFormat.init(this, SAVE, "pixelFormat");
	bitPerPixel.init(this, SAVE, "bitPerPixel");
	version.init(this, NOSAVE, "version");
	video.init(this, SAVE, "video");

	/*
	 * Init the device command list
	 */
	initCommand(driver::INIT_COMMAND);
	initCommand(driver::READ_COMMAND);
	initCommand(driver::SYNCHRONIZE_READ_COMMAND);
	initCommand(driver::START_COMMAND);
	initCommand(driver::RESUME_COMMAND);
	initCommand(driver::PAUSE_COMMAND);
	initCommand(driver::STOP_COMMAND);
	initCommand(driver::STATUS_COMMAND);
	initCommand(driver::READ_INFOS_COMMAND);
	initCommand(DetectorCommon::CLEAR_COMMAND);
	initCommand(DetectorCommon::REGROUP_COMMAND);
	initCommand(AcquisitionCommon::WRITE_PARAMETERS_COMMAND);

	// Init functions
	registerParentFunction(NONE_FUNCTION);
	registerFunction(NONE_FUNCTION);

	deviceType = ROOT_DEVICE_TYPE_DEVICE_CONTAINER;
	m_SynchroniseReadActivated = false;
	m_SynchroniseReadTerminated = false;
	m_DataShort = NULL;

	registerObserverCommand(COMMAND_STATUS_DEVICE_CONTAINER, 50);

}

/*
 * Destructor
 */
Dfk31Au03Driver::~Dfk31Au03Driver() {
}

/*
 * refreshDataSizeProperty
 */
void Dfk31Au03Driver::refreshDataSizeProperty(int32 value) throw (CannotSetValue) {
	if (m_DataShort != NULL) {
		delete[] m_DataShort;
	}
	m_DataShort = new uint8[value];
	if (m_DataShort == NULL) {
		sendErrorEvent(NOTENOUGH_MEMORY_ERROR, "Can't create data table");
		throw CannotSetValue();
	}
	DetectorCommon::refreshDataSizeProperty(value);
}

/*
 * refreshModeProperty
 */
void Dfk31Au03Driver::refreshModeProperty(bool value) throw (CannotSetValue) {

}

/*
 * execute
 */
void Dfk31Au03Driver::execute(const std::string& aCommand) {

	if ((aCommand != driver::PAUSE_COMMAND) && (aCommand != driver::STOP_COMMAND) && (aCommand != driver::STATUS_COMMAND)
			&& (aCommand != driver::READ_COMMAND)) {
		commandProgression = PROGRESSION_UNKNOWNSTATE_DEVICE_CONTAINER;
	}
	Dfk31Au03State* currentState = dynamic_cast<Dfk31Au03State*>(getCurrentState());
	// Check command
	if (aCommand == driver::INIT_COMMAND) {
		// Init command
		currentState->init();
	} else if (aCommand == driver::READ_COMMAND) {
		// Read command
		currentState->read();
	} else if (aCommand == driver::SYNCHRONIZE_READ_COMMAND) {
		// Read command
		m_SynchroniseReadActivated = true;
		m_SynchroniseReadTerminated = false;
		currentState->synchroniseRead();
	} else if (aCommand == driver::START_COMMAND) {
		// Start command
		startActivated = true;
		currentState->start();
	} else if (aCommand == driver::RESUME_COMMAND) {
		// Continue command
		currentState->resume();
	} else if (aCommand == driver::PAUSE_COMMAND) {
		// Stop command
		currentState->pause();
	} else if (aCommand == driver::STOP_COMMAND) {
		// Stop command
		currentState->stop();
		startActivated = false;
	} else if (aCommand == driver::STATUS_COMMAND) {
		// Status command
		currentState->readStatus();
	} else if (aCommand == driver::READ_INFOS_COMMAND) {
		// Info command
		currentState->readInfos();
	} else if (aCommand == DetectorCommon::CLEAR_COMMAND) {
		// Move command
		sum = 0;
		currentState->clear();
	} else if (aCommand == AcquisitionCommon::WRITE_PARAMETERS_COMMAND) {
		// Write command
		currentState->writeParam();
	} else {
		//Error bad command
		//Todo , normaly it will be detect it device action driver
	}
}

/*
 * Dfk31Au03Driver
 */
void Dfk31Au03Driver::computeDetectorStatus(int32 value) {
	int32 valueout = 0;
	detectorStatus.update(valueout);

}

/*
 * Dfk31Au03Driver
 */
void Dfk31Au03Driver::computeAcquisitionStatus(int32 value) {
	int32 valueout = 0;
	acquisitionStatus.update(valueout);
}

}

